import datetime
import time

import BeautifulSoup
import requests

class BugNotFound(Exception):
    pass

class BugNotAvailable(Exception):
    pass

DEFAULT_MIN = 1
DEFAULT_MAX = 999999
DEFAUT_STEP = 20000

def print_distros_dates():
    print_archlinux_dates()
    print_debian_dates()
    print_redhat_dates()
    print_ubuntu_dates()

def print_archlinux_dates():
    MIN = DEFAULT_MIN
    MAX = DEFAULT_MAX
    STEP = 5000
    print_dates("arch", archlinux_date, MIN, MAX, STEP)


def print_debian_dates():
    MIN = 20000
    MAX = DEFAULT_MAX
    STEP = 720001
    print_dates("debian", debian_date, MIN, MAX, STEP)


def print_redhat_dates():
    MIN = DEFAULT_MIN
    MAX = DEFAULT_MAX
    STEP = DEFAUT_STEP
    print_dates("redhat", redhat_date, MIN, MAX, STEP)


def print_ubuntu_dates():
    MIN = DEFAULT_MIN
    MAX = 1225000
    STEP = DEFAUT_STEP
    print_dates("ubuntu", ubuntu_date, MIN, MAX, STEP)


def print_dates(distrib, get_date, min_bug, max_bug, step):
    for bug_id in xrange(min_bug, max_bug, step):
        try:
            print_date(distrib, bug_id, get_date)
            time.sleep(3)
        except (BugNotAvailable, BugNotFound) as e:
            next_bug_id = bug_id
            while True:
                try:
                    next_bug_id += 1
                    print_date(distrib, next_bug_id, get_date)
                    time.sleep(3)
                    break
                except (BugNotAvailable, BugNotFound) as e:
                    pass
                except:
                    print 'errorWhile', distrib, bug_id

        except:
            print 'error', distrib, bug_id

    

def print_date(distrib, bug_id, get_date):
    date = get_date(bug_id)
    print distrib, bug_id, date.strftime("%Y %m %d")

def archlinux_date(bug_id):
    """https://bugs.archlinux.org/task/36714

    <div id="fineprint">
	 Attached to Project:
	 <a href="/index.php?project=1">Arch Linux</a>
	 <br>
	 Opened by <a href="https://bugs.archlinux.org/user/562">Luke McCarthy (shaurz)</a>     	 - Wednesday, 28 August 2013, 22:39 GMT 	   </div>

    """
    r = requests.get("https://bugs.archlinux.org/task/%s" % bug_id)
    parsed_html = BeautifulSoup.BeautifulSoup(r.text)
    try:
        date = parsed_html.find(id="fineprint").contents[-1].split(", ")[1]
    except AttributeError:
        raise BugNotFound()
    return datetime.datetime.strptime(date, "%d %B %Y")


def debian_date(bug_id):
    """
    http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=20000

    <p>Date: Tue, 19 Dec 2000 23:04:10 UTC</p>"""
    payload = {'bug': str(bug_id)}
    r = requests.get("http://bugs.debian.org/cgi-bin/bugreport.cgi", params=payload)
    try:
        r.raise_for_status()
    except requests.exceptions.HTTPError:
        raise BugNotFound()
    parsed_html = BeautifulSoup.BeautifulSoup(r.text)
    try:
        #date = parsed_html.findAll("div", {"class":"header"})[-1].contents[-1]
        _parags = parsed_html.find("div", {"class":"buginfo"}).findAll('p')
        for p in _parags:
            if p.contents[0].startswith('Date:'):
            s = p.contents[0]
            break
        date = s.replace('Date: ', "").replace(' UTC', '')
    except IndexError:
        raise BugNotAvailable()
    date_without_timezone = " ".join(date.split()[:-1])
    try:
        return datetime.datetime.strptime(date_without_timezone, "%d %b %Y %H:%M:%S")
    except ValueError:
        return datetime.datetime.strptime(date_without_timezone, "%a, %d %b %Y %H:%M:%S")

def redhat_date(bug_id):
    """https://bugzilla.redhat.com/show_bug.cgi?id=919040

    <td>2013-03-07 08:24 EST by <span class="vcard redhat_user"><span class="fn">Lukas Zachar</span>
    """
    payload = {'id': str(bug_id)}
    r = requests.get("https://bugzilla.redhat.com/show_bug.cgi", params=payload)
    parsed_html = BeautifulSoup.BeautifulSoup(r.text)
    try:
        if parsed_html.title.string in (u'Access Denied', u'Invalid Bug ID'):
            raise BugNotAvailable()
        date = " ".join(parsed_html.find(id="bz_show_bug_column_2").tr.td.contents[0].split()[:-2])
    except AttributeError:
        raise BugNotFound()
    return datetime.datetime.strptime(date, "%Y-%m-%d %H:%M")


def ubuntu_date(bug_id):
    """https://bugs.launchpad.net/bugs/308191
    
    <span title="2008-12-15 14:52:18 UTC">on 2008-12-15</span>
    """
    r = requests.get("https://bugs.launchpad.net/bugs/%s" % bug_id)
    try:
        r.raise_for_status()
    except requests.exceptions.HTTPError:
        raise BugNotFound()
    parsed_html = BeautifulSoup.BeautifulSoup(r.text)
    date = parsed_html.find(id="registration").find("span")["title"]
    return datetime.datetime.strptime(date, "%Y-%m-%d %H:%M:%S %Z")


if __name__ == '__main__':
    print_distros_dates()

